import SwiftUI
import Collections
import HealthKit
import HKDatabase
import HealthKitExtensions
import CoreLocation

struct WorkoutDetailView: View {
    
    let workout: Workout

    private let store = HKHealthStore()

    @State
    var heartRateSamples: [HeartRate] = []

    @State
    var heartRateSamplesInDatabase: [HeartRate] = []

    @State
    var locationSamples: [CLLocation] = []



    var body: some View {
        List {
            Section("Info") {
                DetailRow("ID", value: workout.id)
                DetailRow("Total Distance", kilometer: workout.totalDistance)
                DetailRow("Goal", value: workout.goal)
            }
            if !workout.activities.isEmpty {
                Section("Activities") {
                    ForEach(workout.activities, id: \.startDate) { activity in
                        NavigationLink(value: activity) {
                            DetailRow(activity.workoutConfiguration.activityType.description,
                                      date: activity.startDate)
                        }
                        
                    }
                }
            }
            if !workout.events.isEmpty {
                Section("Events") {
                    NavigationLink(value: workout.events) {
                        DetailRow("Events", value: workout.events.count)
                    }
                }
            }
            if !workout.metadata.isEmpty {
                Section("Metadata") {
                    NavigationLink {
                        WorkoutMetadataView(metadata: workout.metadata)
                    } label: {
                        DetailRow("Metadata", value: workout.metadata.count)
                    }
                }
            }

            Section("Heart Rate") {
                DetailRow("Count", value: "\(heartRateSamples.count)")
                DetailRow("Range", value: "\(heartRateSamples.minimumHeartRate) - \(heartRateSamples.maximumHeartRate)")
                DetailRow("Database count", value: "\(heartRateSamplesInDatabase.count)")
                DetailRow("Database range", value: "\(heartRateSamplesInDatabase.minimumHeartRate) - \(heartRateSamplesInDatabase.maximumHeartRate)")
            }

            if !locationSamples.isEmpty {
                Section("Locations") {
                    DetailRow("Count", value: "\(locationSamples.count)")
                }
            }
        }
        .navigationTitle(workout.typeString)
        .navigationDestination(for: HKWorkoutActivity.self) { activity in
            ActivityDetailView(activity: activity)
        }
        .navigationDestination(for: [HKWorkoutEvent].self) {
            WorkoutEventsView(events: $0)
        }
        .onAppear(perform: loadSamples)
    }

    private func loadSamples() {
        Task {
            do {
                let samples = try await self.loadHeartRateData()
                DispatchQueue.main.async {
                    self.heartRateSamples = samples
                }
                print("Loaded \(samples.count) heart rate samples")
            } catch {
                print("Failed to load heart rate samples: \(error)")
            }
            do {
                let samples = try self.queryDatabase()
                DispatchQueue.main.async {
                    self.heartRateSamplesInDatabase = samples
                }
                print("Loaded \(samples.count) heart rate samples from database")
            } catch {
                print("Failed to load heart rate samples from database: \(error)")
            }
        }
    }

    private func loadHeartRateData() async throws -> [HeartRate] {
        let sort = SortDescriptor<HKQuantitySample>.init(\.endDate, order: .forward)

        guard let start = workout.firstActivityDate,
              let end = workout.activities.compactMap({ $0.endDate }).max() else {
            print("No dates to get heart rates")
            return []
        }

        print("Heart rates from \(start) to \(end)")
        let predicate = HKQuery.predicateForSamples(
            withStart: start,
            end: end,
            options: [])

        return try await store.read(
            predicate: predicate,
            sortDescriptors: [sort],
            limit: nil)
    }

    func queryDatabase() throws -> [HeartRate] {
        guard let start = workout.firstActivityDate,
              let end = workout.activities.compactMap({ $0.endDate }).max() else {
            print("No dates to get heart rates")
            return []
        }

        return try HealthDatabase.shared.samples(from: start, to: end)
    }
}

#Preview {
    HealthDatabase.shared = .mock()
    return NavigationStack {
        WorkoutDetailView(workout: .mock1)
    }
}

extension String: Identifiable {
    
    public var id: Self { self }
}

private extension Array where Element == HeartRate {

    var minimumHeartRate: Int {
        self.min {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }

    var maximumHeartRate: Int {
        self.max {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }
}

private extension HeartRate {

    var beatsPerMinute: Int {
        quantity.doubleValue(for: .count().unitDivided(by: .minute())).roundedInt
    }
}
